#import "ImageBrowserController.h"
#import "ImageBrowserItem.h"
#import "ImageView.h"
#import "DirectoryBrowser.h"
#import "Utility.h"


@implementation ImageBrowserController

@synthesize window = mWindow;
@synthesize imageView = mImageView;
@synthesize imageBrowser = mImageBrowser;
@synthesize directoryBrowser = mDirectoryBrowser;

#pragma mark init & dealloc

-(void)awakeFromNib
{
    mImages = [[NSMutableArray alloc] init];
    mImportedImages = [[NSMutableArray alloc] init];

	// configure the image browser aspect
	[mImageBrowser setIntercellSpacing:NSMakeSize(5.0f, 5.0f)];
	NSDictionary * titleAttributes = [NSDictionary dictionaryWithObjectsAndKeys:[NSColor whiteColor], NSForegroundColorAttributeName, nil];
	[mImageBrowser setValue:titleAttributes forKey:IKImageBrowserCellsTitleAttributesKey];
	[mImageBrowser setAllowsMultipleSelection:YES];

	// bind the background color of the image browser to the user defaults
	NSMutableDictionary * options = [[NSMutableDictionary alloc] init];
	[options setObject:[NSNumber numberWithBool:YES] forKey:NSContinuouslyUpdatesValueBindingOption];
	[options setObject:NSKeyedUnarchiveFromDataTransformerName forKey:NSValueTransformerNameBindingOption];
	[mImageBrowser bind:IKImageBrowserBackgroundColorKey toObject:[NSUserDefaultsController sharedUserDefaultsController] withKeyPath:@"values.backgroundColor" options:options];
	[options release];
}

-(void)dealloc
{
	[mImages release];
	[mImportedImages release];
	[mBackgroundColor release];
	[super dealloc];
}

#pragma mark Handling of the path

-(NSString *)path
{
	return mPath;
}

-(void)setPath:(NSString *)path
{
	if (path != mPath)
	{
		// copy the new path
		[mPath release];
		mPath = [path copy];

		// reset the selection
		[mWindow setTitle:mPath];
		[mImageView setImageWithURL:nil];
		[mImages removeAllObjects];

		// load the new images in a separate thread
		[NSThread detachNewThreadSelector:@selector(addImagesInPath:) toTarget:self withObject:mPath];
	}
}

-(void)addImagesInPath:(NSString *)path
{
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	
	// get the content of the directory
	NSArray * content = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:path error:nil];
	
	// for each item, create an image object and add to the mImportedImages array
	for (NSString * p in content)
	{
		// ignore files beginning with '.'
		if ([[p lastPathComponent] characterAtIndex:0] == '.')
			continue;
		
		// complete file path
		NSString * file = [path stringByAppendingPathComponent:p];
		
		// ignore non-images and non-directory files
		BOOL isDirectory;
		if ([[NSFileManager defaultManager] fileExistsAtPath:file isDirectory:&isDirectory] == YES && isDirectory == NO)
		{
			NSString * fileType = [Utility humanReadableFileType:file];
			if ([fileType rangeOfString:@"JPEG"].length == 0 &&
				[fileType rangeOfString:@"GIF"].length == 0 &&
				[fileType rangeOfString:@"PNG"].length == 0 &&
				[fileType rangeOfString:@"TIFF"].length == 0 &&
				[fileType rangeOfString:@"DNG"].length == 0 &&
				[fileType rangeOfString:@"CR2"].length == 0)
				continue;
		}			

		ImageBrowserItem * image = [[ImageBrowserItem alloc] init];
		[image setPath:file];
		[image setTitle:[file lastPathComponent]];
		[image setIsDirectory:isDirectory];
		[mImportedImages addObject:image];
		[image release];
	}
	
	// when loading is done, update the data source in the main thread
	[self performSelectorOnMainThread:@selector(pathUpdated) withObject:nil waitUntilDone:YES];
	
	[pool release];
}

-(void)pathUpdated
{
	[self updateDatasource];
	[mImageBrowser scrollIndexToVisible:0];
}


#pragma mark IKImageBrowserDataSource implementation

-(void)updateDatasource
{
    [mImages addObjectsFromArray:mImportedImages];
    [mImportedImages removeAllObjects];
    [mImageBrowser reloadData];
}

-(NSUInteger)numberOfItemsInImageBrowser:(IKImageBrowserView *)view
{
    return [mImages count];
}

-(id)imageBrowser:(IKImageBrowserView *)view itemAtIndex:(NSUInteger)index
{
    return [mImages objectAtIndex:index];
}


#pragma mark IKImageBrowserDelegate

-(void)imageBrowserSelectionDidChange:(IKImageBrowserView *)aBrowser
{
	// get the selection indexes
	NSIndexSet * selectionIndexes = [aBrowser selectionIndexes];
	
	// if there is no image selected, revert to default state
	if ([selectionIndexes count] == 0)
	{
		[mWindow setTitle:mPath];
		[mImageView setImageWithURL:nil];
		return;
	}

	NSString * path = [[mImages objectAtIndex:[selectionIndexes firstIndex]] path];
	[mImageView setImageWithURL:[NSURL fileURLWithPath:path]];		
	[mWindow setTitle:[path lastPathComponent]];
}

-(void)imageBrowser:(IKImageBrowserView *)aBrowser cellWasDoubleClickedAtIndex:(NSUInteger)index
{
	// get the first selected item
	NSIndexSet * selectionIndexes = [aBrowser selectionIndexes];
	ImageBrowserItem * item = [mImages objectAtIndex:[selectionIndexes firstIndex]];

	// if it's a directory select it in the directory browser view
	if ([item isDirectory] == YES)
		[mDirectoryBrowser selectChild:[[item path] lastPathComponent]];
	
	// else, launch the fullscreen view
	else
		[mImageView toggleFullscreen];
}

#pragma mark utilities

-(NSArray *)selectedImages
{
	NSIndexSet * selectedIndexes = [mImageBrowser selectionIndexes];
	return [mImages objectsAtIndexes:selectedIndexes];
}

@end
